#!/usr/bin/env python
#################################################################################
# setBrowser.py
# used in conjunction with the runBrowser shell script
#
# c2008-2015 by Paul Sherman <psherman2001@gmail.com>     
#									        
# This program is free software; you can redistribute it and/or			
# modify it under the terms of the GNU General Public License			
# as published by the Free Software Foundation; either version 3		
# 										
# This program is distributed in the hope that it will be useful,		
# but WITHOUT ANY WARRANTY; without even the implied warranty of		
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the			
# GNU General Public License for more details.					
#										
# You should have received a copy of the GNU General Public License		
# along with this program; if not, write to the Free Software			
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.   
#################################################################################
# set the default browser on Absolute Linux
#********************************************************************************

import os, re, sys, gtk, subprocess
cfile = os.path.join(os.path.expanduser("~/"), ".bashrc")
icefile = os.path.join(os.path.expanduser("~/"), ".icewm/toolbar")

firefox_icon='prog "Web Browser" "firefox" runBrowser'
chrome_icon='prog "Web Browser" "chrome" runBrowser'

class bchooser:

	def alert(self, message):
		box = gtk.MessageDialog(None, 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, message)
		box.set_position(gtk.WIN_POS_CENTER)
		box.set_title('Error')
		box.run()
		box.destroy()	

	# This is a callback functionthe quits the program.
	def delete_event(self,widget,event,data = None):
		gtk.main_quit()
		return False	


	def doClose(self, w=None):
		gtk.main_quit()

	def changed_browser(self, evt=None, data=None):
		browser = self.combo.get_active_text()
		#
		# Set browser env. variable via user .bashrc ---------------------------------
		#
		whole = ""
		matched = ""
		browserline = 'export BROWSER=' + browser + '\n'
		file = open(cfile)
		while 1:
			line = file.readline()
			if not line:
				break
			if re.match('export BROWSER', line):
				line = browserline
				matched = "yes"
			elif line == "\n":
				continue
			whole = whole + line
		file.close()

		if not matched == "yes":
			whole = whole + browserline
		nfile = open(cfile, "w")
		nfile.write(whole)
		nfile.close()
		
		# update mime and desktop databases
		try:
			cmd1 = "update-mime-database /usr/share/mime 2>/dev/null"
			cmd2 = "update-desktop-database 2>/dev/null"
			if "firefox" in browser:
				oldstr = 'chrome'
				newstr = 'firefox'
			if "chrome" in browser:
				oldstr = 'firefox'
				newstr = 'chrome'
			cmd4 = "killall -HUP icewm& > /dev/null 2>&1"
			os.system(cmd1)
			os.system(cmd2)
			subprocess.call(['sed', '-i', 's/'+oldstr+'/'+newstr+'/', icefile])
			os.system(cmd4)
		except:
			pass

		box = gtk.MessageDialog(None, 0, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, "Default browser has been set.\nThis will take effect upon your next login.")
		box.set_position(gtk.WIN_POS_CENTER)
		box.set_title('Success')
		box.run()
		box.destroy()
		gtk.main_quit()

	def __init__(self):		
		# set dropdown selections here
		brows = []
		os.chdir('/usr/share/applications')
		dFiles = os.listdir("/usr/share/applications")
		for df in dFiles: 
			try:
				f=open(df, "r")
				fileString = f.read()
				f.close()

				if re.search("MimeType=text/html", fileString):
					bname = os.path.splitext(df)[0]

					if (bname != "html2pdf") and (bname != "htmlpage") and (bname != "runBrowser") and (not "calibre" in bname):
						brows.append(bname)
			except:
				pass

		# Create a new window.
		self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
		
		# Set the title of the window to "Combo Box".
		self.window.set_title("Web Browser")

		# Set a handler for delete_event that immediately exits GTK.
		self.window.connect("delete_event", self.delete_event)

		# Set the border width of the window.
		self.window.set_border_width(16)


		###########  layout  ################
		vbox = gtk.VBox(False, 16)
		self.window.add(vbox)
		
		bot_box = gtk.HBox(True,0)
		btn_close = gtk.Button("Cancel")
		btn_close.connect("clicked", self.doClose)
		bot_box.pack_start(btn_close, False, False, 0)
		
		btn_set = gtk.Button(" Set ")
		btn_set.connect("clicked", self.changed_browser)
		bot_box.pack_start(btn_set, False, False, 0)
		
		lbl_about = gtk.Label("Choose the Web browser\nyou would like to be the default")
		
		# Create a combo box.
		self.combo = gtk.combo_box_new_text()
		for l in brows:
			self.combo.append_text(l)
		self.combo.set_active(0)

		vbox.pack_start(self.combo, False, False, 0)
		vbox.pack_start(lbl_about, False, False, 0)
		vbox.pack_end(bot_box, False, False, 0)
		btn_close.show()
		btn_set.show()
		bot_box.show()	
		self.combo.show()
		lbl_about.show()
		vbox.show()
		self.window.show()
		
		

	def main(self):
		gtk.main()
		
		
if __name__ == "__main__" :
	widget = bchooser()
	widget.main()
