#!/usr/bin/perl
############################################################################
# addremove.pl
# version 1.2
# c2009-2010 by Paul Sherman                                                    
# <psherman2001@gmail.com>                          
#									        
# This program is free software; you can redistribute it and/or			
# modify it under the terms of the GNU General Public License			
# as published by the Free Software Foundation; either version 3		
# of the License, or (at your option) any later version.			
#										
# This program is distributed in the hope that it will be useful,		
# but WITHOUT ANY WARRANTY; without even the implied warranty of		
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the			
# GNU General Public License for more details.					
#										
# You should have received a copy of the GNU General Public License		
# along with this program; if not, write to the Free Software			
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.   
#############################################################################
# based upon code by Paul Malcher, called "Slackware GUI Packager Manager"
# I adapted code to include txz packages, avoid erroring out,
# UI improvements and refuses to install packages already installed.
# written for Absolute Linux, requires perl-TK
#############################################################################
use Tk;
use Tk::DialogBox;
use Tk::TextUndo;

$ENV{'PATH'} = '/bin:/usr/bin:/sbin';

$main = MainWindow->new(-title=> "Add/Remove Packages", -borderwidth=> 8);
$menubar = $main->Frame()->pack(-side => "top", -fill => "x");
$main->resizable(1,0);
$main->geometry("640x600+0+0");

# define the contents and variable of some dialog boxes
$about = $main->DialogBox(-title=>"About",-buttons=>["OK"]); #Creates the about Dialog
$aabout = $about->add("Label",-text=>"original source by Paul Malcher\nAdapted for Absolute by Paul Sherman\n\nuses perl-tk as a front-end\nfor \"installpkg\" and \"removepkg\"\nand just generally make listing and \nviewing package info a bit easier.\n\nVersion-1.1")->pack;

$ahelp = $main->DialogBox(-title=>"Add/Remove Help",-buttons=>["OK"]); #Creates the help Dialog
$aahelp = $ahelp->add("Label",-justify=>"left",-text=>"\n\"Install a Package\"\n  lets you browse your system\n  for TXZ and TGZ packages to install.\n\n\"Remove Package\"\n  Will remove the currently\n  selected package from your system.\n\n\Double-Click\n  On a selected package\n  to view the package information.\n")->pack;


# File menu
$filemenu = $menubar->Menubutton(-text => 'File', -underline => 0,-tearoff => 0)->pack(-side=>'left');
$filemenu->command(-label => 'Exit',-command => sub{exit});

$aboutmenu = $menubar->Menubutton(-text => 'Help', -underline => 0,-tearoff => 0)->pack(-side=>'left');
$aboutmenu->command(-label => 'About',-command => sub{$about->Show;});
$aboutmenu->command(-label => 'Help',-command => sub{$ahelp->Show;});

# defines the gui layout
$pkglistl = $main->Label(-text=>"Installed Package List \nDouble Click on a package to see a discription\n")->pack();
$pkglist = $main->Listbox(-relief => 'flat', -fg=> 'gray12', -background=>'gray79', -selectforeground=> 'RoyalBlue', -selectbackground=> 'seashell2', -height=>20, -borderwidth=>12, -font=> "Sans 18 bold");
$scroll = $pkglist->Scrollbar(-command => ['yview', $pkglist], -width=>18);
$pkglist->configure(-yscrollcommand => ['set', $scroll]);
$pkglist->pack(-fill => 'both', -ipady => 100, -expand => 1);
$scroll->pack(-side=>'right',-fill => 'both');
$pkginfol = $main->Label(-text=>'Package Info')->pack();
$pkginfo = $main->Scrolled(TextUndo,-height => 11, -scrollbars=>'osoe',-background=>'white', -wrap => 'none',-state=>'disabled')->pack(-fill=>'both',-expand=>1);

# buttons
$pkglb = $main->Button(-text=>'Exit',-command => sub{exit})->pack(-side=>'right');

$pkgrmb = $main->Button(-text=>'Remove Package',-command => sub{
$sfile = $pkglist->get(active);
$cdel = $main->DialogBox(-title=>"Package Removal",-buttons=>["Remove","Cancel"]);
$cdel->add("Label",-text=>"\nWarning you are about to unistall:\n\n$sfile\n\n     Are you sure you want to do this?      \n")->pack;
$result = $cdel->Show;


if($result eq "Remove"){

	$label->configure(-text=>'Removing... please wait', -foreground => 'red');
	$label->update;
	system("/sbin/removepkg $sfile");
	$label->configure(-text=>"$sfile REMOVED", -foreground => 'black');
	$label->update;
	&pkglistr();
	$main->geometry("640x600+0+0");
}else{
	$label->configure(-text=>'Unistall Aborted!', -foreground => 'black');
	$label->update;
	&pkglistr();
	}
})->pack(-side=>'right');

$pkgaddb = $main->Button(-text=>'Install a Package',-command => sub{
	if($open eq ""){
	my $types = [['Package TXZ', '*.txz'],['Package older TGZ', '*.tgz']];
	my $open = $main->getOpenFile(-filetypes=>$types);
	$longname = substr($open, 0, length($open)-4);
	@fArray = split(/\//, $longname);
	$fName = $fArray[-1];
	$curname = "/var/log/packages/" . $fName;
	if (-e $curname){
		$err = $main->DialogBox(-title=>"Error",-buttons=>["Ok"]);
		$err->add("Label",-text=>"Error: That package is already installed")->pack;
		$err->Show;
	} else {
	$label->configure(-text=>'Installing... please wait', -foreground => 'red');
	$label->update;
	system("installpkg $open");
	$label->configure(-text=>"$fName INSTALLED", -foreground => 'black');
	$label->update;
	&pkglistr();
	$pkglist->update;
	# send us to index of newly installed package
	@items = $pkglist->get(0,'end');
	$id = 0;
	$nIndex = 0;
	foreach (@items) {
		if ($_ eq $fName){
			$nIndex = $id;
			last;
		} else {
			$id++;
		}
    }
	$pkglist->yview($nIndex);
	}}
})->pack(-side=>'right');

$label = $main->Label(-text=>'')->pack(-side=>'bottom');

$pkglist->bind('<Double-1>' , \&loaddesc);
#$pkgrmb->tooltip("Click to remove\nthe currently selected package")
&pkglistr();

sub loaddesc {
	$pf = $pkglist->get(active);
	$pkginfo->configure(-state=>'normal');
	$pkginfo->delete('1.0','end');
	$flag = 0;
	open(desc,"</var/log/packages/$pf");
	@lines = <desc>;
	close desc;
	foreach(@lines){
	if($_ eq "FILE LIST\:\n"){
	$flag = 1;
	}
	if($flag ne "1"){
	$pkginfo->insert('end',"$_");
	}
	}
	$pkginfo->configure(-state=>'disabled');
}

# user checker, alert the person running the app if not root
if($> ne "0"){
	$cuid = $main->DialogBox(-title=>"Effective User Id Error",-buttons=>["Ok"]);
	$cuid->add("Label",-text=>"Warning, you will not be able to install/uninstall applications. I must run from root or setuid!")->pack;
	$cuid->Show;
	$admin = 1;
	$pkgaddb->configure(-state=>'disabled');
	$pkgrmb->configure(-state=>'disabled');
	}else{
	$admin = 0;
}

sub pkglistr {
	opendir(DIR, "/var/log/packages") or warn ("Cannot open current directory! Autoload Aborted!");
	my @contents = readdir(DIR);
	closedir(DIR);
	# Links may be ignored completely:
	# No hidden files and ".." directories:
	# for we run setuid properly data must be untainted before Tk's use,was a pain to figure out

	$count = 0;
	while(@contents[$count] ne ""){
	@contents[$count] =~ /[\/\w\-._]+/;             # includes 'file' chars + "\"
	$temp = $&;
	@contents[$count] = "$temp";
	$count++;
	}

	@contents = grep {!/^\./} @contents;
	@contents = sort @contents; # lexical sort
	$pkglist->delete(0,'end');
	foreach(@contents){
	$pkglist->insert(end,$_);
	}
}

MainLoop; #The main processing loop
